package net.jaekl.frank;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.concurrent.ConcurrentHashMap;

import net.jaekl.qd.QDBundleFactory;

public class FrankBundle {
	public static final String ANSWER_RECEIVED = "answer.received";
	public static final String DATA_COLLECTED = "data.collected";
	public static final String DESTINATION = "destination";
	public static final String ERROR_PAGE = "error.page";
	public static final String ETA = "eta";
	public static final String FRANK = "frank";
	public static final String GPS_OFF = "gps.off";
	public static final String GPS_READ = "gps.read";
	public static final String HIDE_DETAILS = "hide.details";
	public static final String INVALID_RESPONSE = "invalid.response";
	public static final String MAYBE_SERVER_PROBLEM = "maybe.server.problem";
	public static final String MINUTES = "m";	// suffix (abbreviated) for minutes
	public static final String REMAIN = "remain";
	public static final String REQUEST_MADE = "request.made";
	public static final String RETURN_TO_MAIN_PAGE = "return.to.main.page";
	public static final String ROUTE = "route";
	public static final String SECONDS = "s";
	public static final String SERVER_TIMEOUT = "server.timeout";
	public static final String SHOW_DETAILS = "show.details";
	public static final String UNEXPECTED_ERROR = "unexpected.error";
	public static final String UNEXPECTED_EXCEPTION = "unexpected.exception";
	public static final String URL_CONTACTED = "url.contacted";
	
	final static String BUNDLE_NAME = "frank";
	
	static ConcurrentHashMap<Locale, FrankBundle> m_bundleMap = new ConcurrentHashMap<Locale, FrankBundle>();
	
	ResourceBundle m_bundle;
	
	public static FrankBundle getInst(Locale locale) {
		FrankBundle result = m_bundleMap.get(locale);
		if (null == result) {
			synchronized(FrankBundle.class) {
				result = m_bundleMap.get(locale);
				if (null == result) {
					result = new FrankBundle(locale); 
				}
				m_bundleMap.put(locale, result);
			}
		}
		return result;
	}
	
	private FrankBundle(Locale locale) {
		m_bundle = QDBundleFactory.getInst().getBundle(BUNDLE_NAME, locale); 
	}
	
	public String get(String key) {
		try {
			if (null != m_bundle) {
				return m_bundle.getString(key);
			}
		}
		catch (MissingResourceException e) {
			// Make it clear that something has gone wrong.
			e.printStackTrace();	
			// Fall through to the fallback behaviour below
		}
		return "[" + key + "]";
	}
}
