package net.jaekl.frank;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.jaekl.frank.octranspo.Server;
import net.jaekl.frank.octranspo.StopInfo;
import net.jaekl.qd.util.ExceptionUtils;

public class ViewSchedule extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	static final String BUNDLE_NAME = "frank";
	static final String STOP = "stop";
	static final String ROUTE = "route";
	static final String LANG = "lang";
	static final String UTF_8 = "UTF-8";
	
	ErrorHandler m_errorHandler;
	
	public ViewSchedule() {
		m_errorHandler = new ErrorHandler();
	}
	
	int getParamInt(HttpServletRequest req, String paramName) {
		String valueStr = getParamString(req, paramName);
		try {
			return Integer.parseInt(valueStr);
		}
		catch (NumberFormatException exc) {
			// TODO:  Error page
			return 0;
		}
	}
	
	String getParamString(HttpServletRequest req, String paramName) {
		String valueStr = req.getParameter(paramName);
		if (null == valueStr|| valueStr.equals("")) {
			// TODO:  Error page
			return null;
		}
		return valueStr;
	}
	
	Locale getLocale(HttpServletRequest req) {
		Locale result = null;
		String lang = getParamString(req, LANG);
		if (null != lang) {
			result = new Locale(lang);
		}
		if (null == result) {
			result = Locale.getDefault();
		}
		return result;
	}

	@Override
	public void doGet(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		res.setContentType("text/html");
		res.setCharacterEncoding("utf8");	// Seems to be necessary when running in Japanese
		PrintWriter pw = res.getWriter();
		
		Locale locale = getLocale(req);
		int stop = getParamInt(req, STOP);
		int route = getParamInt(req, ROUTE);
		
		try {
			FileInputStream fis = new FileInputStream("apikey.txt");
			try {
				Server server = new Server("192f31d2", fis);
				StopInfo stopInfo = null;
				
				if (0 == route) {
					stopInfo = server.getNextTripsForStopAllRoutes(stop);
				}
				else {
					stopInfo = server.getNextTripsForStop(stop, route);
				}
				Schedule schedule = new Schedule(locale);
				
				schedule.writePage(pw, stopInfo);
			}
			finally {
				ExceptionUtils.tryClose(fis);
			}
		}
		catch (Throwable t) {
			m_errorHandler.writeErrorPage(pw, t, locale);
		}
	}
}
